#!/usr/bin/env python
# Nicolas Franco Sierra 11/nov/2016
# extract_RNAs.py
# Extracts rRNAs and ITSs for each GB record in a multiGB file 
# output: a FASTA file for each feature
# usage: python extract_genes.py seqs_file.gb

from Bio import SeqIO
import sys

input_genbank = str(sys.argv[1])
product_key = ["18S ribosomal RNA", "internal transcribed spacer 1", "5.8S ribosomal RNA", "internal transcribed spacer 2", "28S ribosomal RNA"]
filename_dict = {"18S ribosomal RNA":["18S.fasta",0], "internal transcribed spacer 1":["ITS1.fasta",0], "5.8S ribosomal RNA":["5_8S.fasta",0], "internal transcribed spacer 2":["ITS2.fasta",0], "28S ribosomal RNA":["28S.fasta",0]}

try:
    f = {gname:open(fname[0],"w") for gname,fname in filename_dict.iteritems()}
    genbank_handle = open(input_genbank, "rU")
    for rec in SeqIO.parse(genbank_handle, "genbank"):
        if rec.features:
            for feature in rec.features:
                if feature.type == "misc_RNA" or feature.type == "rRNA":
                    if feature.qualifiers != None and feature.qualifiers.get('product') != None and feature.qualifiers['product'][0] in product_key: ##(Revisar)
                        f[str(feature.qualifiers['product'][0])].write(">" + str(rec.annotations['organism']).replace(" ","_").replace(".","").replace("-","_") + "\n")           	     
                        f[str(feature.qualifiers['product'][0])].write(str(feature.location.extract(rec).seq) + "\n")
                        seq_len = len(feature.location.extract(rec).seq)
                        if seq_len > filename_dict[feature.qualifiers['product'][0]][1]:
                            filename_dict[feature.qualifiers['product'][0]][1] = seq_len
finally:
    for fh in f.values():
        fh.close()


for val in filename_dict.values():
    print val[1]


try:
    f = [filelist for filelist in filename_dict.values()]
    fout = [open(gname[0].replace(".fasta", "_filtered.fasta"), "w") for gname in filename_dict.values()]
    gene_dict = {}
    for fin,fo in zip(f,fout):
    #for fin in f:
        i_filename = fin[0]
        max_lenght = fin[1]
        gene_dict[i_filename] = ""
        sp_dict = {}
        seq_dict = {}
        fasta_handle = open(i_filename, "rU")
        for rec in SeqIO.parse(i_filename, "fasta"):
            if len(rec.seq) >= int(0.65 * max_lenght):
                sp_name = str(rec.id)
                if sp_name in sp_dict:
                    sp_dict[sp_name] += 1
                    seq_dict[sp_name].append(rec.seq)
                else:
                    sp_dict[sp_name] = 1
                    seq_dict[sp_name] = [rec.seq]
                fo.write(">" + str(rec.id) + "\n")
                fo.write(str(rec.seq) + "\n")
        #gene_dict[i_filename].append([])
        fasta_handle.close()
finally:
    for fo in fout:
        fo.close()


            
